/* scanner for generating x2x translators */

%{
#include "x2x.h"
#include "gen_x2x.tab.h"

#include <math.h>
#include <string.h>
#define YY_NO_UNPUT
int yyparse();
%}
 
DIGIT    [0-9]
ID       [a-zA-Z_][a-zA-Z0-9_]*
QUOTE_CHAR \\.

%x comment
%x sentence
%x string
%x perl
%%

"skeleton_version_1" {return SKELETON_VERSION_1;}

"/*"   {BEGIN(comment); }
<comment>[^*\n]*        /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   /* eat up '*'s not followed by '/'s */
<comment>\n             ++line_num;
<comment>"*"+"/"        BEGIN(INITIAL);

"{" {BEGIN(sentence); return '{';}
<sentence>\\\n[ \t]*\\ {++line_num;}
<sentence>[^}\\]+ {CSTR; return STRING;}
<sentence>{QUOTE_CHAR} {CSTR; return STRING;}
<sentence>"}" {BEGIN(INITIAL); return '}';}

"\"" {BEGIN(string); return '"';}
<string>[^\"]+ {CSTR; return STRING;}
<string>"\"" {BEGIN(INITIAL); return '"';}

"%perl {" {BEGIN(perl); return PERL_BEGIN;}
<perl>\n {line_num++; CSTR; return STRING;}
<perl>[^%\n]+ {CSTR; return STRING;}
<perl>"%"[^\n}]+ {CSTR; return STRING;}
<perl>"%}" {BEGIN(INITIAL); return PERL_END;}

"or" {return OR;}

{ID} {CSTR; return NAME;}

"<=>" {return TRANSLATION;}

"output.c" {return OUTPUT_C;}

[:{}\|;\.] {return yytext[0];}

[ \t]+ /*{CSTR; return WHITE_SPACE;}*/
\n          line_num++;
.           {fprintf(stderr,"%s:%i: Unrecognized character: %s\n",
                     parse_fname,line_num,yytext);}

%%

int main(int argc, char **argv)
{
  ++argv, --argc;  /* skip over program name */
  if ( argc > 0 ) {
    CMEM(parse_fname=strdup(argv[0]));
    yyin = fopen( parse_fname, "r" );
    if(!yyin) {
      perror(parse_fname);
      exit(-1);
    }
  } else {
    CMEM(parse_fname=strdup("stdin"));
    yyin = stdin;
  }

  init();

  yyparse();
  return 0;
}
