/* common for c2txt and txt2c */

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include "x2x.h"

int line_num=1;
char *parse_fname=NULL;

int yyerror(char *str)
{
  puts(str);
  return 0;
}

struct names_st
{
  char *name,*id,*ntype;
} *names=NULL;
int names_count;

char *C_txt,*C_add;

//return name or exits
char *add_name(char *name)
{
  if(names_count>=MAX_NAMES) {
    fprintf(stderr,"MAX_NAMES too small\n");
    exit(-1);
  }
  names[names_count].id=NULL;
  return names[names_count++].name=strdup(name);
}

char *get_id(char *name)
{
  int i;
  for(i=0;i<names_count && names[i].id;i++) 
    if(strcmp(names[i].name,name)==0) return names[i].id;
  fprintf(stderr,"get_name: Ooops: id not found for name %s\n",name);
  exit(-1);
  return NULL;
}

char *get_name(char *id)
{
  int i;
  for(i=0;i<names_count && names[i].id;i++) 
    if(strcmp(names[i].id,id)==0) return names[i].name;
  fprintf(stderr,"get_name: Ooops: name not found for id %s\n",id);
  exit(-1);
  return NULL;
}

char *get_type(char *id)
{
  int i;
  for(i=0;i<names_count && names[i].id;i++) 
    if(strcmp(names[i].id,id)==0) return names[i].ntype;
  fprintf(stderr,"get_type: Ooops: name not found for id %s\n",id);
  exit(-1);
  return NULL;
}

// returns name that corresponds to id
char *add_id(char *id, char *ntype)
{
  int i;
  for(i=0;i<names_count;i++) {
    if(!names[i].id) {
      names[i].id=strdup(id);
      names[i].ntype=strdup(ntype);
      D(printf("added:%s",id));
      return names[i].name;
    } else if(strcmp(names[i].id,id)==0) {
      if(strcmp(names[i].ntype,ntype)) {
        D(printf("different types: id=%s, %s -> %s",id,names[i].ntype,ntype));
        free(names[i].ntype);
        names[i].ntype=strdup(ntype);
      }
      return names[i].name;
    }
  }
  fprintf(stderr,"Not enough names\n");
  exit(-1);
  return NULL;
}

int read_names(char *file_name)
{
#define NAME_LEN 100
  int i;
  char str[NAME_LEN];
  FILE *fp=fopen(file_name,"r");
  if(!fp) {
    if(!fp) perror(file_name);
    return -1;
  }
  CMEM(names=malloc(sizeof(names[0])*MAX_NAMES));
  names_count=0;

  while(fgets(str,NAME_LEN,fp)) {
    i=strlen(str);
    if(i>0 && str[i-1]=='\n') str[i-1]=0;
    add_name(str);
  }
  fclose(fp);
  return 0;
}  

#if DEBUG
char *debStr;
#endif

char *nstr2hexstr(char *num_str)
{
  static char str[20];
  sprintf(str,"0x%08lx",strtoul(num_str,NULL,10));
  return str;
}

/*9876543210*/
/*2244738638*/
char poem_words[10][10][20]={
  {"day.", "second!", "full moon?", "year.", "week.", "month.", "hour.",
   "minute.", "picosecond?","terasecond!"},
  {"every", "once in", "always except every", "during this", "sometime this",
   "this", "first half of", "middle of", "part of", "several times during"},
  {"the moon", "the sun", "the earth", "mountain", "molehill", "dust particle",
   "lava", "star", "monitor", "itself"},
  {"over", "under", "through", "alongside", "left of", "right of",
   "in the midst of", "among", "without", "within"},
  {"jumps", "walks", "licks", "dances", "glazes", "whispers", "sits", "pokes",
   "stacks", "rotates"},
  {"happily", "fortunately", "sadly", "ordinarily", "passionately",
   "violently", "kindly", "angrily", "unluckily", "gently"},
  {"box", "ball", "hat", "poem", "lamp", "pot", "car", "truck", "dog", "cat"},
  {"red", "green", "blue", "yellow", "orange", "purple", "brown", "black",
   "grey", "tan"},
  {"big", "small", "large", "tiny", "petite", "huge", "humongous", "great",
   "little", "diminutive"},
  {"Very", "Extremely", "Slightly", "Rather", "Somewhat", "Especially",
   "Particularly", "Unusually", "Exceptionally", "Unexceptionally"},
  /* 0000000000 = Very big red box happily jumps over the moon every day.*/
};

char *poem2nstr(char *poem)
{
  static char res[11];
  int i,j;
  char *s=poem;
  for(i=9;i>=0;i--)
    for(j=0;j<10;j++)
      if(strncmp(s,poem_words[i][j],strlen(poem_words[i][j]))==0) {
        res[9-i]=j+'0';
        s+=strlen(poem_words[i][j])+1;
      }
  res[10]=0;
  return res;
}

char *num2poem(unsigned long int n)
{

  static char poem[200];
  char tmp[200];
  unsigned long int digit;
  int i;
  poem[0]=0;
  for(i=0;i<=9;i++) {
    strcpy(tmp,poem);
    digit=n%10;n/=10;
    sprintf(poem,"%s %s",poem_words[i][digit],tmp);
  }
  
  return poem;
}

unsigned long int hex2n(char *hex_str)
{
  int len=strlen(hex_str);
  if(len<3) return 0;
  if(hex_str[len-1]=='l') hex_str[len-1]=0;
  return strtoul(hex_str,NULL,16);
}

char *hex2nstr(char *hex_str)
{
  static char hex_buffer[13];
  sprintf(hex_buffer,"%lu",hex2n(hex_str));
  return hex_buffer;
}

#define ALLOC_DEBUG !FALSE
char *astr(int count, ...)
/* copy count number strings together and return result */
/* arguments are freed and result is allocated */
{
  char *res;
  int res_len,len=0;
  char *s;
  int i,l;
  va_list args;
  res_len=30;
  CMEM(res=malloc(res_len));
  strcpy(res,"");
  va_start(args, count);
  for(i=0;i<count;i++) {
    s=va_arg(args,char*);
    l=strlen(s);
    while(len+l+1>res_len) {
      res_len*=2;
      CMEM(res=realloc(res,res_len));
    }
    strcpy(res+len,s);
    len+=l;
  }
  va_end(args);
  if(yydebug) fprintf(stderr,"astr:%s\n",res);
  return res;
}

void free_chk(void *ptr, char *msg)
{
  if(yydebug) fprintf(stderr,"free_chk(%s): <%s>\n",msg,(char*)ptr);
  free(ptr);
}
